"""
QR: Redis-Based Data Structures in Python

         5 Mar 2010 | Returns work correctly for both bounded and unbounded
	 4 Mar 2010 | Pop commands now return just the value
	24 Feb 2010 | QR now has deque and stack data structures (0.1.2)
	22 Feb 2010 | First public release of QR (0.1.1)
"""

__author__ = 'Ted Nyman'
__version__ = '0.1.3'
__license__ = 'MIT'

import redis

try:
    import json
except ImportError:
    import simplejson as json

#The redis-py object -- modify/remove this to match with your namespacing
redis = redis.Redis()
	
#The Deque
class Deque(object):

	#Key is required; specify a size to get a bounded deque
	def __init__(self, key, size=None):
		self.key = key
		self.size = size		

	#Push to Back
	def pushback(self, element):
		key = self.key
		length = redis.llen(key)

		if length == self.size:
			popped = redis.rpop(key)
			push_it = redis.lpush(key, element)
			print 'PUSHED: %s' % (element)
			return popped 
		
		else:
			push_it = redis.lpush(key, element)
			print 'PUSHED: %s' % (element)
		
	#Push to Front
	def pushfront(self, element):
		key = self.key
		length = redis.llen(key)

		if length == self.size:
			popped = redis.lpop(key)
			push_it = redis.rpush(key, element)
			print 'PUSHED: %s' % (element)
			return popped 
		
		else:
			push_it = redis.rpush(key, element)
			print 'PUSHED: %s' % (element)

	#Pop Front Element
	def popfront(self):
		key = self.key
		popped = redis.rpop(key)
		return popped 


	#Pop Back Element
	def popback(self):
		key = self.key
		popped = redis.lpop(key)
		return popped 


	#Return all elements from the deque as a Python list
	def elements(self):
		key = self.key
		length = redis.llen(key)
		all_elements = redis.lrange(key, 0, length)
		return all_elements
				
	#Return all elements from the deque as a JSON object
	def elements_as_json(self):
		key = self.key
		size = self.size
		length = redis.llen(key)
		all_elements = redis.lrange(key, 0, length)
		all_elements_as_json = json.dumps(all_elements)
		return all_elements_as_json

#The Queue
class Queue(object):	
	
	#Key is required; specify a size to get a bounded queue
	def __init__(self, key, size=None):
		self.key = key
		self.size = size		

	#Push
	def push(self, element):
		key = self.key
		length = redis.llen(key)

		if length == self.size:
			popped = redis.rpop(key)
			push_it = redis.lpush(key, element)
			print 'PUSHED: %s' % (element)
			return popped 
		
		else:
			push_it = redis.lpush(key, element)
			print 'PUSHED: %s' % (element)
		
	#Pop 
	def pop(self):
		key = self.key
		popped = redis.rpop(key)
		return popped 	

	#Return all elements from the queue as a Python list
	def elements(self):
		key = self.key
		length = redis.llen(key)
		all_elements = redis.lrange(key, 0, length)
		return all_elements
				
	#Return all elements from the queue as a JSON object
	def elements_as_json(self):
		key = self.key
		length = redis.llen(key)
		all_elements = redis.lrange(key, 0, length)
		all_elements_as_json = json.dumps(all_elements)
		return all_elements_as_json

#The Stack
class Stack(object):

	#Key is required; specify a size to get a bounded stack
	def __init__(self, key, size=None):
		self.key = key
		self.size = size		

	#Push
	def push(self, element):
		key = self.key
		length = redis.llen(key)

		if length == self.size:
			popped = redis.lpop(key)
			push_it = redis.lpush(key, element)
			print 'PUSHED: %s' % (element)
			return popped 
		
		else:
			push_it = redis.lpush(key, element)
			print 'PUSHED: %s' % (element)
		
	#Pop 
	def pop(self):
		key = self.key
		popped = redis.lpop(key)
		return popped 
	
	#Return all elements as a Python list
	def elements(self):
		key = self.key
		length = redis.llen(key)
		all_elements = redis.lrange(key, 0, length)
		return all_elements
					
	#Return all elements as a JSON object
	def elements_as_json(self):
		key = self.key
		length = redis.llen(key)
		all_elements = redis.lrange(key, 0, length)
		all_elements_as_json = json.dumps(all_elements)
		return all_elements_as_json



#!/usr/bin/env python

from setuptools import setup, find_packages

version = '0.1.3'

LONG_DESCRIPTION = '''

QR
--

QR helps you create and work with deque, queue, and stack data structures for Redis. 

Redis is well-suited for implementations of these abstract data structures, and QR makes the work even easier in Python. QR works best for (and simplifies) the creation of bounded deques, queues, and stacks (herein, DQS's), with a defined size of elements.

Quick Setup
-----------

You'll need Redis itself, and the current Python interface for Redis, redis-py. Put qr.py in your PYTHONPATH and you're all set.

qr.py also creates an instance of the redis-py interface object. You may already have instantiated the object in your code, so you'll want to ensure consistent namespacing. You can remove this line of code, modify the namespacing, or adjust your existing namespacing -- whatever works best for you.

Full documentation is in README.md, or at http://github.com/tnm/qr

'''


setup(
    name='qr',
    version=version,
    description='Create and interact with Redis-based data structures in Python',
    long_description=LONG_DESCRIPTION,
    url='http://github.com/tnm/qr',
    author='Ted Nyman',
    author_email='tnm800@gmail.com',
    keywords='Redis, queue, data structures',
    license='MIT',
    packages=find_packages(),
    py_modules=['qr'],
    include_package_data=True,
    zip_safe=False,
    classifiers=[
	'Programming Language :: Python',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
   ],
)
